/* A second test program of the FlagShip/JLT API
 *
 * Browse the data in the part table of the abc database.
 * Edit the quantity of the chosen record.
 *
 * To COMPILE: FlagShip fstest1.prg fs_sql.o sql_error.o -o\ fstest1 -lsql
 */

#include "fileio.fh"
#include "inkey.fh"

// Retrieved records (i.e., tmp files) will be written in this directory.
// Make certain this directory already exits and that the user that runs
// this program has read/write permission in this directory. 
//                                      Edit to suit your system 
//                                      here.
FS_SET("setenvir", "JLT_RETRIEVE_DIR", "/home/griffin/SQLkit/")

// Open a database and check for errors.
LOCAL nRetVal := open_jlt( "abc" )
sql_error( nRetVal, "fstest1", "open_jlt")

// Do NOT lock the records retrieved below. Allow this program to read
//  records locked by other users.
nRetVal := sql_unlock()
sql_error( nRetVal, "fstest1", "sql_unlock")

// retrieve the parts table
cRelname := "part"

// Since we want all the records, set the where clause to NIL.
// Records will be returned in their default order, i.e., by part_no.
cWclause := NIL

// If you want them ordered by name, use this as the "where clause".
// cWclause := "name like '%' order by name"

// Variables used below.
LOCAL cTfilename, nFieldCount, nRowCount

// Retrieve the records and check for errors.
nRowCount := sql_ret(cRelname, cWclause , @cTfilename, @nFieldCount)
sql_error( nRowCount, "fstest1", "sql_ret")

// Variables used to read the data from the temporary file, etc.
LOCAL text, handle
LOCAL_INT nC1, nC2, nBarPos

// This two-dimensional array will hold each data element retrieved.
LOCAL aBrow [nRowCount] [nFieldCount]

// read the temporary file
handle := FOPEN( cTfilename, FO_READ )
FOR nC1 := 1 TO nRowCount
  // Read a line of text equivalent to a single record.
  text := FREADTXT( handle )
  // Parse each record into its separate fields; each field ends with a bar.
  FOR nC2 := 1 TO nFieldCount
      nBarPos := AT( "|", text )
      aBrow[nC1][nC2] := SUBSTR( text, 1, nBarPos-1 )
      text := SUBSTR( text, nBarPos+1 )
  NEXT
NEXT

FCLOSE( handle )             /* close the temporary file */
FERASE( cTfilename )         /* delete the temporary file */ 

// Get the data into a one-dimensional array of string... easy to 
// display using ACHOICE(). Or, you could create a Browse object and
// load aBrow into it, in which case you would not need
// aMyarray, etc.
LOCAL aMyarray[nRowCount]

FOR nC1 := 1 TO nRowCount
  aMyarray[nC1] := PADR(aBrow[nC1][1], 7) + PADR(aBrow[nC1][2], 23) + ;
        PADR(aBrow[nC1][3], 5)
NEXT

// Display screen starts here.
@ 1, 1 SAY "ABC Auto Parts Co.                                     Browse/Edit parts table"
LOCAL cPCount := STR(LEN(aBrow)) + " Parts" 
@ 3, 3 SAY cPCount

@ 6, 7 SAY "ID     Name                   Quantity"

LOCAL nChoice := ACHOICE(8, 7, 20, 45, aMyarray)

// Since aMyarray and aBrow are parallel, nChoice now holds
// the index of the record you have chosen to update.

// Allow user to update the quantity for the selected record.
BEGIN SEQUENCE
  IF .NOT. nChoice == 0

    // Since we are about to update the chosen record, get a lock on it
    // by selecting (retrieving) it again.

    // Turn on record locking.
    nRetVal := sql_lock()
    sql_error(nRetVal, "fstest1", "sql_lock")

    // Lock the record.
    cWclause := "part_no = " + aBrow[nChoice][1]
    nRetVal := sql_aret("part", cWclause)
    sql_error(nRetVal, "fstest1", "sql_aret")

    @ 3, 3 CLEAR TO MAXROW(), MAXCOL() 
    @ 3, 3 SAY aMyarray[nChoice]
    LOCAL cNewQuantity := SPACE(4)
    @ 5, 19 SAY "New Quantity:" GET cNewQuantity PICTURE "@! 9999"
    READ

    // User may ESC without affecting the database.
    IF LASTKEY() == K_ESC
      BREAK
    ENDIF

    cNewQuantity := ALLTRIM( cNewQuantity )

    // If the user hits Enter on an empty cNewQuantity field, a poorly
    // formed SQL command would hang the system, so an empty field must
    // be intrepreted as a zero!
    IF LEN( cNewQuantity ) == 0
      cNewQuantity := "0"
    ENDIF

    // Update the record, check for errors.
    LOCAL AtValStr := "quantity = " + cNewQuantity
    cWclause := "part_no = " + aBrow[nChoice][1]
    nRetVal := sql_repl( "part", AtValStr, cWclause )
    sql_error( nRetVal, "fstest1", "sql_repl")

    // Make the user agree to the update
    DO WHILE .T.
      @ 7, 7 SAY "OK to update the database? (y/n): "
      LOCAL nAnswer := INKEY(0)
      IF CHR(nAnswer) == "y" .OR. CHR(nAnswer) == "Y"
        ?? "Y"
        nRetVal := sql_commit()
        sql_error(nRetVal, "fstest1", "sql_commit" )
        @ 9, 7 SAY "Database updated."
        INKEY(3)
        BREAK
      ENDIF
      IF CHR(nAnswer) == "n" .OR. CHR(nAnswer) == "N" 
        ?? "N"     
        nRetVal := sql_rback()
        sql_error( nRetVal, "fstest1", "sql_rback" )
        @ 9, 7 SAY "Database not updated."
        INKEY(3)
        BREAK
      ENDIF
      ?? CHR(7)  /* none of the above, beep at the user */
    ENDDO
  ENDIF
END

// Close the database, and we're done.
nRetVal := close_jlt()
sql_error( nRetVal, "fstest1", "close_jlt")

// That's all, folks!

